
#ifndef CHROMOSOME_HPP
#define CHROMOSOME_HPP

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
/* a Chromosome is a part of a warrior; each of these parts can have different
   settings etc */
	
#include "species.hpp"
#include <string>

class CChromosome {
	friend class CWarrior;
	public:
		// construction
		CChromosome(CWarrior *warrior,const unsigned int idx,const unsigned int len=0);
		// getters
		CWarrior *warrior() const { return _warrior; }
		CChromosomeType *type() const { return _type; }
		unsigned int idx() const { return _idx; }
		unsigned int len() const { return _len; }
		//insn_t &operator[](unsigned i) const { return code(i); }
		insn_t &code(unsigned i) const;
		const std::string &info() const { return _info; }
		bool is_start() const;
		unsigned start() const; // returns MAXLENGTH if warrior start is not in this chromosome
		// setters
		void set_len(const unsigned int len);
		void rnd_len();
		void clear(); // invalidates body
		void set_start(const unsigned i); // actually sets the appropriate stuff in the warrior
		std::string &info() { return _info; }
		// safety
		void check(const field_t coresize) const; // panics if not ok
	private:
		CWarrior *_warrior;
		CChromosomeType *_type;
		unsigned int _idx, _len, _ofs;
		void calc_ofs();
		std::string _info;
};
	
class CChromosomeType {
	public:
		// construction
		CChromosomeType(CSpecies *species); // create an auto-chromosome shim
		CChromosomeType(CSpecies *species,const char *name); // create a user-ready shim
		// serialisation
		void read_ini(INIFile &ini);
		void write_ini(std::ostream &os);
		// getters
		CSpecies *species() const { return _species; }
		const char *name() const { return _name.c_str(); }
		bool is_auto() const { return _auto; }
		CLength *length() const { return _length; }
		CInstGenerator *freq() const { return _freq; }
		COperand *operands() const { return _operands; }
		CReproduction *reproduction() const { return _reproduction; }
	private:
		bool _auto; // is this just a shim, nothing local?
		std::string _name;
		CSpecies *_species;
		void init();
		CLength *_length; // might be CGenus::_length if not overriden
		CInstGenerator *_freq; // might be CGenus::_freq if not overriden
		COperand *_operands; // might be CGenus::_operands if not overriden
		CReproduction *_reproduction; // might be CGenus::_reproduction if not overriden
};

#endif // CHROMOSOME_HPP

