
/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "distribution.hpp"
#include "error.hpp"
#include "rand.hpp"

using namespace std;

/***** CDistribution class implementation *************/

CDistribution::CDistribution() {
	_context = "";
	_data = 0;
	_size = 0;
}

CDistribution::CDistribution(const string &context,const int min,const int max) {
	_context = context;
	_data = 0;
	_size = 0;
	set(min,max);
}

CDistribution::~CDistribution() {
	if(0 != _data) // allocated?
		delete[] _data;
}

void CDistribution::set(const int min,const int max) {
	if(min >= max)
		PANIC(MISC,"distribution has invalid length",NULL);
	_size = 2;
	if(0 != _data) // already allocated?
		delete[] _data;
	_data = new int[_size];
	_data[0] = min;
	_data[1] = max;
}

void CDistribution::read_ini(INIFile &ini,bool required) {
	KeyValuePair *kvp;
	int i;
	string num;
	// get size
	kvp = ini.get(_context + "_num_points");
	if(0 == kvp)
		if(required) {
			PANIC(MISC,_context.c_str(),"_num_points expected")
		} else {
			return;
		}
	_size = kvp->getValueAsInt();
	if(_size < 2)
		PANIC(MISC,_context.c_str(),"invalid _num_points");
	// allocate
	delete[] _data;
	_data = new int[_size];
	// and load
	for(i=0; i<_size; ++i) {
		// prepare num
		num = _context;
		num += '_';
		string_append(num,i);
		// get key
		kvp = ini.get(num); if(0 == kvp) PANIC(MISC,num.c_str(),"expected");
		// and set it
		_data[i] = kvp->getValueAsInt();
		// check sequence
		if(i > 0) { // not the first?
			if(_data[i] <= _data[i-1])
				PANIC(MISC,num.c_str(),"value out of sequence");
		}
	}
}

CDistribution *CDistribution::read_override_ini(INIFile &ini,ImplFactoryCallback factory) {
	CDistribution *ret = this;
	KeyValuePair *kvp;
	int i;
	string num;
	// get size
	kvp = ini.get(_context + "_num_points");
	if(0 == kvp)
		return ret; // nothing specified?
	ret = factory(*this);
	ret->_size = kvp->getValueAsInt();
	if(ret->_size < 2)
		PANIC(MISC,_context.c_str(),"invalid _num_points");
	// allocate
	ret->_data = new int[ret->_size];
	// and load
	for(i=0; i<ret->_size; ++i) {
		// prepare num
		num = _context;
		num += '_';
		string_append(num,i);
		// get key
		kvp = ini.get(num); if(0 == kvp) PANIC(MISC,num.c_str(),"expected");
		// and set it
		ret->_data[i] = kvp->getValueAsInt();
		// check sequence
		if(i > 0) { // not the first?
			if(ret->_data[i] <= ret->_data[i-1])
				PANIC(MISC,num.c_str(),"value out of sequence");
		}
	}
	return ret;
}

void CDistribution::write_ini(ostream &os) {
	int i;
	if(0 == _data) // not allocated?
		PANIC(MISC,_context.c_str(),"distribution not allocated");
	os << _context << "_num_points=" << _size << endl;
	for(i=0; i<_size; ++i) {
		os << _context << '_' << i << "=" << _data[i] << endl;
	}
}

void CDistribution::write_override_ini(ostream &os,const CDistribution *parent) {
	if(this == parent) // not overriden?
		return;
	write_ini(os);
}

int CDistribution::rnd() const { /* return a random number within this distribution */
	const int i = CRand::irand(_size-1); // target value
	int	x = _data[i],
		y = _data[i+1],
		ret = (x + CRand::irand(y - x));
	if(!ok(ret)) // out of range??
		PANIC(BAD_CRand,"beyond distribution range",NULL);
	// done
	return ret;
}
