
/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "error.hpp"

#include <fstream>

namespace { // anon

	const char 
		*CSpeciesError_CODES[CSpeciesError::CODE_LAST+1] = {
			"MISC",
			"MARS",
			"BAD_SPECIES_STATE",
			"BAD_OPCODE",
			"NEG_OPCODEFREQ_HINT",
			"ZERO_OPCODEFREQ_HINT_SUM",
			"BAD_CRand",
			"BAD_FILE",
			"EVOLVING_SPECIES_CANNOT_ADD_BENCHMARK_WARRIOR",
			"BENCHMARK_SPECIES_CANNOT_ADD_EVOLVING_CWarrior",
			"CANNOT_OPEN_GENERATION_FILE",
			"CWarrior_NOT_IN_GENERATION_FILE",
			"GENERATION_FILE_ERROR",
			"MARS_ALLOC_ERROR",
			"CANNOT_OPEN_INI",
			"BAD_GEN_FILE",
			"BAD_RESUMEFILE",
			"BAD_CODE"
		};
} // anon namespace

void string_append(std::string &s,unsigned int i) { // very crude, ought to be a way to sort this out
	const char *number = "0123456789";
	unsigned int x = s.size();
	if(0 == i) { // special case?
		s.insert(x,1,'0');
	} else while(i > 0) {
		s.insert(x,1,number[i % 10]);
		i /= 10;
	}
}

bool file_exists(std::string &s) {
	using namespace std;
	fstream file(s.c_str(),ios::in);
	if(file.is_open()) {
		file.close();
		return true;
	}
	return false;
}

/***** CSpeciesError class implementation ************/

CSpeciesError::CSpeciesError(const CSpeciesError::CODE code,const char *s1,const char *s2) {
	_code = code;
	_desc = "CSpeciesError <";
	_desc += code_desc(_code);
	_desc += "> ";
	if(0 != s1)
		_desc += s1;
	if(0 != s2) {
		if(0 != s1)
			_desc += ": ";
		_desc += s2;
	}
}

const char *CSpeciesError::code_desc(const CSpeciesError::CODE code) {
	if((0 <= code) && (code < CODE_LAST))
		return CSpeciesError_CODES[code];
	return CSpeciesError_CODES[CSpeciesError::CODE_LAST]; // bad
}

