#ifndef __ERROR_HPP__
#define __ERROR_HPP__

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include <string>

class CSpeciesError { /* utility class to throw when an unrecoverable error occurs */
	public:
		enum CODE {
			MISC,
			MARS,
			BAD_SPECIES_STATE,
			BAD_OPCODE,
			NEG_OPCODEFREQ_HINT,
			ZERO_OPCODEFREQ_HINT_SUM,
			BAD_CRand,
			BAD_FILE,
			EVOLVING_SPECIES_CANNOT_ADD_BENCHMARK_WARRIOR,
			BENCHMARK_SPECIES_CANNOT_ADD_EVOLVING_CWarrior,
			CANNOT_OPEN_GENERATION_FILE,
			CWarrior_NOT_IN_GENERATION_FILE,
			GENERATION_FILE_ERROR,
			MARS_ALLOC_ERROR,
			CANNOT_OPEN_INI,
			BAD_GEN_FILE,
			BAD_RESUMEFILE,
			CODE_LAST
		};
		CSpeciesError(const CODE code,const char *s1 = 0,const char *s2 = 0);
		CODE code() const { return _code; }
		std::string desc() const { return _desc; }
		static const char *code_desc(const CODE code); /* fetch a description of this error condition */
	private:
		CODE _code;
		std::string _desc;
};

// easy macros
#define PANIC(x,s1,s2) throw new CSpeciesError(CSpeciesError::x,s1,s2);

// utilities, not strictly error-related, but hey

void string_append(std::string &s,unsigned int i); // very crude, ought to be a way to sort this out

bool file_exists(std::string &s);

#endif // __ERROR_HPP__
