#include "exec_trail.hpp"

#include <iostream>
using namespace std;

/*** COpcodeMap implementation ***/

COpcodeMap::COpcodeMap() {
	makeopmap();
}

unsigned COpcodeMap::opmap(field_t field) const {
	return _opmap[field];
}

void COpcodeMap::mapop(OPCODE op,OPTYPE type) {
	for(unsigned i=0; i<MODIFIER_LAST; i++) {
		_opmap[_OP(op,(MODIFIER)i)] = type;
	}
}

void COpcodeMap::mapop(OPCODE op,MODIFIER mod,OPTYPE type) {
	_opmap[_OP(op,mod)] = type;
}

void COpcodeMap::makeopmap() {
	mapop(DAT,OTHER);
	mapop(SPL,SPLIT);
	mapop(MOV,MATH);
	mapop(DJN,BRANCH);
	mapop(ADD,MATH);
	mapop(JMZ,BRANCH);
	mapop(SUB,MATH);
	mapop(SEQ,BRANCH);
	mapop(SNE,BRANCH);
	mapop(SLT,BRANCH);
	mapop(JMN,BRANCH);
	mapop(JMP,BRANCH);
	mapop(NOP,OTHER);
	mapop(MUL,MATH);
	mapop(MODM,MATH);
	mapop(DIV,MATH);
	mapop(LDP,OTHER);
	mapop(STP,OTHER);
	mapop(MOV,mI,MOVI); // special case
}

/*** CExecTrail implementation ***/

CExecTrail::CExecTrail(COpcodeMap *map): _map(map) {
	_exec_count = new dist_t*[_map->segments()];
	for(unsigned seg=0; seg<_map->segments(); seg++) {
		_exec_count[seg] = new dist_t[_map->optypes()];
		for(unsigned op=0; op<_map->optypes(); op++) {
			_exec_count[seg][op] = ZERO;
		}
	}
}

CExecTrail::~CExecTrail() {
	for(unsigned seg=0; seg<_map->segments(); seg++) {
		delete[] _exec_count[seg];
	}
	delete[] _exec_count;
}

void CExecTrail::clear() {
	for(unsigned seg=0; seg<_map->segments(); seg++) {
		for(unsigned op=0; op<_map->optypes(); op++) {
			_exec_count[seg][op] = ZERO;
		}
	}
}

void CExecTrail::set(unsigned const segment,CExecTrail::dist_t table[MAX_OPCODES]) { // table[MAX_OPCODES]
	clear();
	for(unsigned op=0; op<_map->optypes(); op++) {
		_exec_count[segment][op] = ZERO;
	}
	for(field_t op=0; op<MAX_OPCODES; op++) {
		_exec_count[segment][_map->opmap(op)] += table[op];
	}
}

CExecTrail::dist_t CExecTrail::distance_squared(CExecTrail const *other) const {
	dist_t ret = ZERO;
	for(unsigned seg=0; seg<_map->segments(); seg++) {
		for(unsigned op=0; op<_map->optypes(); op++) {
			dist_t d = _exec_count[seg][op] - other->_exec_count[seg][op];
			ret += (d * d);
		}
	}
	return ret;
}

CExecTrail::dist_t CExecTrail::max_distance_from() const {
	dist_t ret = ZERO;
	for(unsigned seg=0; seg<_map->segments(); seg++) {
		for(unsigned op=0; op<_map->optypes(); op++) {
			dist_t d = _exec_count[seg][op];
			if(d < (_map->segsize() >> 1)) // invert?
				d = _map->segsize() - d;
			ret += (d * d);
		}
	}
	return ret;
}

bool CExecTrail::load_from_file(const char *filename) {  // returns success
	_filename = filename;
	bool ret = false;
	clear();
	dist_t entry[MAX_OPCODES];
	// load the file..
	FILE *f = fopen(filename,"rb");
	if(f) {
		for(unsigned seg=0; seg<_map->segments(); seg++) {
			if(1 == fread(entry,sizeof(entry),1,f)) {
				set(seg,entry);
				ret = true;
			} else {
				cerr << "could not read from \"" << filename << "\" when loading the exec trail" << endl;
				ret = false;
				break;
			}
		}
		// done
		fclose(f);
	} else {
		cerr << "could not open \"" << filename << "\" to load the exec trail" << endl;
	}
	return ret;
}

