/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "fitness.hpp"
#include "error.hpp"
using namespace std;

namespace { // anonymous
	
	const char	
		*CFitness_RESULTS[CFitness::RESULT_LAST] = {
			"win","tie","lose"},
		*CFitness_CATS[CFitness::CAT_LAST] = {
			"attack","defence"};
			
} // end of anonymous namespace

/***** CFitness class implementation ******************/

void CFitness::read_ini(INIFile &ini) { /* loads fitness values; values are REQUIRED */
	int cat, res;
	string key;
	KeyValuePair *kvp;
	for(cat=0; cat<CAT_LAST; ++cat) {
		for(res=0; res<RESULT_LAST; ++res) {
			// prepare key name
			key = MNEMONIC_RESULT((RESULT)res);
			key += '_';
			key += MNEMONIC_CAT((CAT)cat);
			// get it
			kvp = ini.get(key);
			if(0 == kvp)
				PANIC(MISC,"fitness weighting expected",NULL);
			// set it
			_data[(CAT)cat][(RESULT)res] = kvp->getValueAsFloat();
		}
	}
}

CFitness *CFitness::read_override_ini(INIFile &ini) /* loads fitness values; if any fitness values are
	specified, then a copy of this fitness will be made; all unspecified values in the new
	class will be "inherited" from this one, and any specified values will be overriden.  If nothing
	is specified, then this will be returned */ {
	CFitness *ret = this; // assume it is
	int cat, res;
	string key;
	KeyValuePair *kvp;
	for(cat=0; cat<CAT_LAST; ++cat) {
		for(res=0; res<RESULT_LAST; ++res) {
			// prepare key name
			key = MNEMONIC_RESULT((RESULT)res);
			key += '_';
			key += MNEMONIC_CAT((CAT)cat);
			// get it
			kvp = ini.get(key);
			if(0 == kvp)
				continue; // quite ok
			// first time to override?
			if(this == ret) {
				ret = new CFitness(*this); // copy it so that all unoverriden values are 'inherited'
			}
			// set it
			ret->_data[(CAT)cat][(RESULT)res] = kvp->getValueAsFloat();
		}
	}
	return ret;
}

void CFitness::write_ini(ostream &os) {
	int cat, res;
	for(cat=0; cat<CAT_LAST; ++cat) {
		for(res=0; res<RESULT_LAST; ++res) {
			os	<< MNEMONIC_RESULT((RESULT)res) << '_' << MNEMONIC_CAT((CAT)cat)
				<< '=' << _data[(CAT)cat][(RESULT)res] << endl;
		}
	}
}

void CFitness::write_override_ini(ostream &os,const CFitness *parent) {
	if(this == parent) // nothing overriden?
		return;
	int cat, res;
	for(cat=0; cat<CAT_LAST; ++cat) {
		for(res=0; res<RESULT_LAST; ++res) {
			if(parent->_data[(CAT)cat][(RESULT)res] != _data[(CAT)cat][(RESULT)res]) { // overriden?
				os	<< MNEMONIC_RESULT((RESULT)res) << '_' << MNEMONIC_CAT((CAT)cat)
					<< '=' << _data[(CAT)cat][(RESULT)res] << endl;
			}
		}
	}
}

const char *CFitness::MNEMONIC_RESULT(const CFitness::RESULT res) {
	if(0 >= res < CFitness::RESULT_LAST)
		return CFitness_RESULTS[res];
	return 0; // bad
}

const char *CFitness::MNEMONIC_CAT(const CFitness::CAT cat) {
	if(0 >= cat < CFitness::CAT_LAST)
		return CFitness_CATS[cat];
	return 0; // bad
}
