
#ifndef GENUS_HPP
#define GENUS_HPP

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
#include "species.hpp"

class CGenus: public CHookable {
	public:
		CGenus(CKingdom *kingdom,const std::string &name);
		~CGenus();
		void clear(); // removes all species from this genus */
		void read_ini(INIFile &ini); /* expects to be in the correct section */
		void write_ini(std::ostream &os);
		void save();
		void ready(); /* this will blank all fitness metrics preceeding the fighting of a generation */
		int fight(CGenus &enemy); /* this will actually do all the fights for a generation; returns number of rounds fought */
		CWarrior *selection(); /* this will do selection on all the warriors in this genus; returns pointer to the best */
		const std::string name() const { return _name; }
		int num_species() const { return _num_species; }
		int num_evolving_species() const;
		int num_evolved_warriors() const;
		int num_warriors() const; /* returns a trivia number of warriors 'live' in this genus at this time */
		int size() const { return _num_species; }
		CKingdom *kingdom() const { return _kingdom; }
		CFitness *fitness() const { return _fitness; }
		CLength *length() const { return _length; }
		CInstGenerator *freq() const { return _freq; }
		COperand *operands() const { return _operands; }
		CReproduction *reproduction() const { return _reproduction; }
		CWarrior *warrior(const CWarrior::TUid uid) const; /* fetch a particular warrior from this genus; NULL if not present */
		// array access
		CSpecies &species(unsigned index) const;
	protected:
		CSpecies **_species;
		int _num_species;
		CKingdom *_kingdom;
		CFitness *_fitness; // might be referencing CKingdom::_fitness if not overriden
		CLength *_length; // might be referencing CKingdom::_length if not overriden
		CInstGenerator *_freq; // might be referencing CKingdom::_freq if not overriden
		COperand *_operands; // might be referencing CKingdom::_operands if not overriden
		CReproduction *_reproduction; // might be referencing CKingdom::_reproduction if not overriden
		std::string _name;
};

#endif // ifndef GENUS_HPP
