
#ifndef __INI_HPP__
#define __INI_HPP__

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */


#include <iostream>
#include <string>

/**
 *
 * this INI parsing thingy is designed to work with binary streams,
 * coping internally with either \n or \r\n line endings
 *
 **/

/*******************************************************************
 * class KeyValuePair
 * sequentially parses lines and splits them into key-value pairs
 * to parse, >> from an input stream in a loop while isValid()
 *******************************************************************/

class KeyValuePair {
   friend std::istream &operator>>(std::istream &is,KeyValuePair &kvp) { return kvp.fromSerial(is); }
   public:
      KeyValuePair();
      ~KeyValuePair();
      // error states
      enum KVP_ERROR {
      	VALUE_NOT_INT,
      	VALUE_NOT_BOOL,
      	VALUE_NOT_FLOAT,
      	VALUE_NOT_DOUBLE,
      	KEY_NOT_VALID,
      	KEY_HAS_NO_CATEGORY,
      	KVP_ERROR_LAST
      };
      static const char *ERROR_MSG(const KVP_ERROR err);
      // something to check?
      bool isValid() const { return valid; }
      // is a key?
      bool is(const std::string &key) const { return is(key.c_str()); }
      bool is(const char *key) const;
      // get key
      void getKeyAsString(std::string &key); // will set key to error message if not a valid line
      const char *getKeyAsChar(); // copy to your own var before you manipulate or save for later!
      // keys can be categorised by being of the form "category.key" so "guidence.range" makes
      // more sense
      bool hasCategory() const { return hascategory; } // simply returns if has a category
      bool isCategory(const std::string &category) const { return isCategory(category.c_str()); }
      bool isCategory(const char *category);
      bool hasSubCategory(); // more categories?  category.category...key
      bool getSubCategory(); // will divide the current key into category.key; returns hasCategory() now
      // get key; will throw KVP_ERROR in error conditions
      void getCategoryAsString(std::string &category); 
      const char *getCategoryAsChar() const; // copy to your own var before you manipulate or save for later!
			// check values
			bool isValue(const std::string &val) const { return isValue(val.c_str()); }
			bool isValue(const char *val) const;
      // get value (where a default is set, will return def if parse failed); else will throw KVP_ERROR
      void getValueAsString(std::string &value);
      const char *getValueAsChar(); // copy your own var before manipulate or save for later!
      int getValueAsInt(int def=0);
      float getValueAsFloat(float def=0.0f,bool percent_allowed=true);
      double getValueAsDouble(double def=0.0,bool percent_allowed=true);
      bool getValueAsBool(bool def=true);
      // misc
      unsigned int getLineNum() const { return linenum; }
      static const int MAX_KVP_LINE_LEN = 132;
   protected:
      bool valid, // should key and value be used?
         hascategory; // has a category?
      unsigned int linenum;
      char line[MAX_KVP_LINE_LEN], *key, *value, *category;
      std::istream &fromSerial(std::istream & is);
};

class INIFile {
	public:
		INIFile(std::istream &is);
		bool seek(const std::string &section); // returns success
		KeyValuePair *get(const std::string &key); // returns null if no such key
	protected:
		static const int NO_SECTION = -1;
		std::istream *_is;
		int _section_ofs;
		char _section[KeyValuePair::MAX_KVP_LINE_LEN];
		KeyValuePair _kvp;
		bool _bad_stream;
};

#endif // ifndef __INI_HPP__

