#ifndef __INST_GEN_WEIGHTED_RANDOM_HPP__
#define __INST_GEN_WEIGHTED_RANDOM_HPP__

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
#include "inst_gen.hpp"

class CInstGeneratorWeightedRandom: public CInstGenerator { /* manages instruction frequency */
	public:
		CInstGeneratorWeightedRandom(); /* constructor creates default values */
		float get_freq_hint(const OPCODE opcode);
		void set_freq_hint(const OPCODE opcode,const float val); 
		// actual query points
		virtual void suggest_instruction(insn_t &instruction,const MODE mode); /* this uses the freq_hint table to pick a random opcode */
		// and saving
		virtual void write_ini(std::ostream &os);
		virtual void write_override_ini(std::ostream &os,const CInstGenerator *parent);
	protected:
		static const int
			OPCODE_FREQ_HINT_DEFAULT = 1,
			BRANCH_PIVOT_DEFAULT = 1;
		bool _branch_bias; /* do we have a bias towards branches being not too adjacent? */
		int _branch_pivot; /* this is the number of non-branch instructions at which branch strength is exact */
		float _freq_hint[OPCODE_LAST], /* a hint to creating random warriors, on the freq of opcodes expected in this genus */
			_freq_hint_sum; /* sum total of all hints in freq_hint table, precalculated as it is used quite often */
		// from CInstGenerator
		virtual void read_ini_impl(INIFile &ini);
		virtual CInstGenerator *read_override_ini_impl(INIFile &ini); /* loads freq values; if any freq values are
			specified, then a copy of this freq will be made; all unspecified values in the new
			class will be "inherited" from this one, and any specified values will be overriden.  If nothing
			is specified, then this will be returned */
};

#endif // __INST_GEN_WEIGHTED_RANDOM_HPP__
