/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "operand.hpp"

// concrete implementation headers
#include "operand_distribution.hpp"
#include "operand_table.hpp"

#include "species.hpp"
#include "rand.hpp"

//#include <string.h>

/************ COperand implementation ****************************/

const char *COperand::IMPL_DESC[COperand::IMPL_LAST] = { "distribution","table" };
const char *COperand::TYPE_KEY = "operands";

const char *COperand::impl_desc(const COperand::IMPL type) {
	if((type >= 0) && (type < IMPL_LAST))
		return IMPL_DESC[type];
	return 0; // bad
}

COperand::IMPL COperand::desc_to_impl(const char *desc) {
	for(int i=0; i<IMPL_LAST; i++) {
		if(strcasecmp(desc,IMPL_DESC[i])==0)
			return (IMPL)i;
	}
	PANIC(MISC,"bad ",type_key());
}

COperand::COperand(const field_t coresize) {
	_coresize = coresize;
}

COperand *COperand::NewDefault(const field_t coresize) {
	return new COperandDistribution(coresize);
}
		
COperand *COperand::read_ini(INIFile &ini,const field_t coresize) {
	KeyValuePair *kvp;
	COperand *ret;
	// identify which implementation, based upon keywords
	kvp = ini.get(type_key());
	if(0 == kvp) { // not specified? we have a default..
		ret = NewDefault(coresize);
	} else {
		switch(desc_to_impl(kvp->getValueAsChar())) {
			case DISTRIBUTION:
				ret = new COperandDistribution(coresize);
				break;
			case TABLE:
				ret = new COperandTable(coresize);
				break;
			default:
				PANIC(MISC,kvp->getValueAsChar(),type_key());
		}
	}
	ret->read_ini_impl(ini);
	return ret;
}

COperand *COperand::read_override_ini(COperand *gen,INIFile &ini) {
	KeyValuePair *kvp;
	COperand *ret;
	// identify which implementation, based upon keywords
	kvp = ini.get(type_key());
	if(0 == kvp) return gen; // nothing to do?
	switch(desc_to_impl(kvp->getValueAsChar())) {
		case DISTRIBUTION:
			if(same_type(gen,ret)) {
				ret =  new COperandDistribution((COperandDistribution&)*gen);
				ret->read_override_ini_impl(ini);
			} else {
				ret = new COperandDistribution(ret->_coresize);
				ret->read_ini_impl(ini);
			}
			break;		
		case TABLE:
			if(same_type(gen,ret)) {
				ret =  new COperandTable((COperandTable&)*gen);
				ret->read_override_ini_impl(ini);
			} else {
				ret = new COperandTable(ret->_coresize);
				ret->read_ini_impl(ini);
			}
			break;		
		default:
			PANIC(MISC,kvp->getValueAsChar(),type_key());
	}
	return ret;
}
