#ifndef __OPERAND_HPP_
#define __OPERAND_HPP_

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
/* to implement a generator:
 * 1) derive publically from COperand
 * 1) give it an identifier in the EImpl enumeration
 * 2) make the constructor set the _type member
 * 3) make COperand factories understand this new implementation and how to identify it
 * 4) implement the read_ini/write_ini functionality
 */
 
#include "ini.hpp"
#include "exhaust.hpp"

class COperand { // an abstract class, use the factory to get an implementation
	public:
		enum Field {
			A,
			B,
			FIELD_LAST
		};
		virtual ~COperand() {}
		// factory
		static COperand *NewDefault(const field_t coresize);
		static COperand *read_ini(INIFile &ini,const field_t coresize); // create a new implementation
		static COperand *read_override_ini(COperand *gen,INIFile &ini); /* loads an implementation;
			if the ini specifies a new implementation: if the type is the same as the parent, it will load a diff;
			if the type is different, all values must be specified */
		// actual query point
		virtual void rnd(insn_t &instruction,const Field field) = 0;
		// and saving
		virtual void write_ini(std::ostream &os) = 0;
		virtual void write_override_ini(std::ostream &os,const COperand *parent) = 0;
		// misc
		static bool same_type(const COperand *gen1,const COperand *gen2) { return (gen1->type() == gen2->type()); } 
	protected:
		COperand(const field_t coresize);
		field_t _coresize;
		enum IMPL { // all implementations must be listed here
			DISTRIBUTION,
			TABLE,
			IMPL_LAST
		} _type;
		static const char *type_key() { return TYPE_KEY; }
		static const char *impl_desc(const IMPL type);
		static IMPL desc_to_impl(const char *desc);
		const IMPL type() const { return _type; }
		virtual void read_ini_impl(INIFile &ini) = 0;
		virtual COperand *read_override_ini_impl(INIFile &ini) = 0;
	private:
		static const char
			*TYPE_KEY,
			*IMPL_DESC[IMPL_LAST];
};

#endif // ifndef __OPERAND_HPP__

