/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
/* to implement a generator:
 * 1) derive publically from COperand
 * 1) give it an identifier in the EImpl enumeration
 * 2) make the constructor set the _type member
 * 3) make COperand factories understand this new implementation and how to identify it
 * 4) implement the read_ini/write_ini functionality
 */
 
#include "operand_distribution.hpp"
#include "error.hpp"

/********** COperandDistribution implementation ******************************/

COperandDistribution::COperandDistribution(const field_t coresize):
	COperand(coresize), CDistribution("operands",0,_coresize) {
	_type = DISTRIBUTION;
}

COperandDistribution::COperandDistribution(const CDistribution &copy):
	CDistribution(copy), COperand(CDistribution::max()) {
	_type = DISTRIBUTION;
}

CDistribution *COperandDistribution::ImplFactory(const CDistribution &copy) {
	return new COperandDistribution(copy);
}


// actual query point
void COperandDistribution::rnd(insn_t &instruction,const Field field) {
	switch(field) {
		case A:
			instruction.a = CDistribution::rnd();
			break;
		case B:
			instruction.b = CDistribution::rnd();
			break;
		default:
			PANIC(MISC,"invalid field in COperandDistribution::rnd",NULL);
	}
}

void COperandDistribution::write_ini(std::ostream &os) {
	os << type_key() << "=" << impl_desc(type()) << std::endl;
	CDistribution::write_ini(os);
}

void COperandDistribution::write_override_ini(std::ostream &os,const COperand *parent) {
	if(parent == this) // nothing to do?
		return;
	os << type_key() << "=" << impl_desc(type()) << std::endl;
	if(same_type(this,parent)) {
		CDistribution::write_override_ini(os,(CDistribution*)parent);
	} else {
		CDistribution::write_ini(os);
	}
}

void COperandDistribution::read_ini_impl(INIFile &ini) {
	CDistribution::read_ini(ini);
}

COperand *COperandDistribution::read_override_ini_impl(INIFile &ini) {
	return (COperand*)CDistribution::read_override_ini(ini,ImplFactory);
}
