
/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "reproduction.hpp"
#include "species.hpp"
#include "rand.hpp"
#include "opcode_branch_lookup.hpp"
using namespace std;

namespace { // anon
		
	const bool FITNESS_BIAS_DEFAULT = false; // a bit of backwards compatibility
	const bool ADJUST_ADDRESSING_DEFAULT = true;

	// this is a YACE style dummy warrior to be killed by an evolving candidate, just to check the evolving candidate has _some_ teeth
	warrior_t runtest_dummy;
	bool runtest_dummy_flag = false; // need to create the runtest_dummy?
		
/*	void randomInstruction(CWarrior &warrior,const int index) {
		warrior.code[index].in = OP(warrior.species()->freq()->get_random_opcode(warrior,index),CRand::irand(MODIFIER_LAST),CRand::irand(ADDRMODE_LAST),CRand::irand(ADDRMODE_LAST));
		warrior.code[index].a = warrior.operands()->rnd();
		warrior.code[index].b = warrior.operands()->rnd();
	} */
	
	/* this check will ensure that there is a SPL, JMP or MOV.I between start and DAT */
	bool loops(const CWarrior &warrior) {
		unsigned int i;
		for(i=warrior.start_idx(); i<warrior.len(); i++) {
			switch(warrior.code(i).opcode()) {
				// oh no, dead!
				case DAT: 
					return false;
				// instructions that (might) loop
				case SPL:
				case DJN:
				case JMZ:
				case JMN:
				case JMP:
					// could check for bad indirects?
					// could discourage JMP-0?
					return true;
				// possible imp?
				case MOV:
					// check it is a .I
					if(mI == warrior.code(i).modifier())
						return true;
					// could check for bad indirects?
				// else continue to next instruction
				default:
					break;
			}
		}
		// if here, doesn't loop
		return false;
	}

	/* this check will run the warrior for a short time in a mars to see if it commits suicide */
	bool runtest(const CWarrior &warrior) {
		const char *RUNTEST_DUMMY_FILENAME = "runtest_dummy.rc";
		const unsigned int
			RUNTEST_DUMMY_LEN = 12,
			RUNTEST_DUMMY_POS = warrior.kingdom()->coresize() >> 1;
		unsigned int i;
		if(!runtest_dummy_flag) {
/*			// header
			runtest_dummy.start=1;
			runtest_dummy.len = RUNTEST_DUMMY_LEN;
			// a djnstream-resistant bit
			i=0;
			runtest_dummy.code[i].in = OP(JMP, mF, IMMEDIATE, DIRECT); // a jmp #n is a wimp
			runtest_dummy.code[i].a = 5 + i; // some colour
			runtest_dummy.code[i].b = 10 + i; // some colour
			// body
			for(i=1; i<(RUNTEST_DUMMY_LEN-1); i++) {
				runtest_dummy.code[i].in = OP(NOP, mF, DIRECT, DIRECT);
				runtest_dummy.code[i].a = 5 + i; // some colour
				runtest_dummy.code[i].b = 10 + i; // some colour
			}
			// loop back to beginning
			i = RUNTEST_DUMMY_LEN-1;
			runtest_dummy.code[i].in = OP(JMP, mF, DIRECT, DIRECT);
			runtest_dummy.code[i].a = warrior.kingdom()->coresize()-runtest_dummy.len+2; // loop
			runtest_dummy.code[i].b = 10 + i; // some colour
			// done
			//discore(runtest_dummy.code,0,runtest_dummy.len,warrior.kingdom()->coresize());/* and print to stdout. */
			cout << "loading dynamic runtest dummy \"" << RUNTEST_DUMMY_FILENAME << "\".." << endl;
			asm_fname(RUNTEST_DUMMY_FILENAME,&runtest_dummy,warrior.kingdom()->coresize());
			runtest_dummy_flag = true;
		}
		sim_clear_core();
		sim_reset_pspaces();
		warrior.load(0);
		sim_load_warrior(RUNTEST_DUMMY_POS,&(runtest_dummy.code[0]),runtest_dummy.len);
		warrior.kingdom()->core_check();
		switch(sim(2,warrior.start_idx(),RUNTEST_DUMMY_POS+runtest_dummy.start,warrior.kingdom()->cycles(),0)) {
			case  0: // warrior won
				return true;
			case 1: // runtest_dummy won
			case 2: // tie
				return false;
			case -1:
			default:
				PANIC(MARS,"runtest","critical mars error");
		}
		return false; // hmm, dumb compiler
	}

} // anon namespace

/***** CReproduction class implementation *************/

const char
	*CReproduction::MUTATION_DESCS[CReproduction::MUTATION_LAST] = {
		"mutation_none","mutation_opcode","mutation_modifier","mutation_addrmode_a","mutation_addrmode_b","mutation_operand_a","mutation_operand_b"},
	*CReproduction::RANDOM_COMPONENT_DESCS[CReproduction::RANDOM_COMPONENT_LAST] = {
		"random_none","random_crossover","random_insert","random_replace","random_delete"},
	*CReproduction::CROSSOVER_DESCS[CReproduction::CROSSOVER_LAST] = {
		"random"};

CReproduction::CReproduction() {
	// at first, mutation is 0 (100% of mutation is in _mutation[NONE])
	int i;
	_crossover = CROSSOVER_RANDOM;
	for(i=0; i<MUTATION_LAST; ++i)
		_mutation[(MUTATION)i] = 0.0F;
	for(i=0; i<RANDOM_COMPONENT_LAST; ++i)
		_crossover_random[(RANDOM_COMPONENT)i] = 0.0F;
	_mutation[MUT_NONE] = 1.0F;
	_mutation_sum = _mutation[MUT_NONE];
	_survive = 1.0F; // never
	_dat_padding = 0.0F; // never
	_fitness_bias = FITNESS_BIAS_DEFAULT;
	_adjust_addressing = ADJUST_ADDRESSING_DEFAULT;
}

void CReproduction::read_ini(INIFile &ini) {
	KeyValuePair *kvp;
	int i;
	// crossover
	kvp = ini.get("crossover"); if(0 == kvp) PANIC(MISC,"crossover expected",NULL);
	_crossover = crossover(kvp->getValueAsChar());
	// crossover-type-specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			_crossover_random_sum = 0.0F;
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				kvp = ini.get(random_component_desc((RANDOM_COMPONENT)i)); if(0 == kvp) PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," expected");
				_crossover_random[(RANDOM_COMPONENT)i] = kvp->getValueAsFloat();
				if(0.0F > _crossover_random[(RANDOM_COMPONENT)i]) PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," cannot be negative");
				_crossover_random_sum += _crossover_random[(RANDOM_COMPONENT)i];
			}
			if(0.0F >= _crossover_random_sum) PANIC(MISC,"sum of random crossover components cannot be negative nor zero",NULL);
			kvp = ini.get("adjust_addressing");
			if(0 != kvp) { // specified?
				_adjust_addressing = kvp->getValueAsBool(ADJUST_ADDRESSING_DEFAULT);
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	kvp = ini.get("survive"); if(0 == kvp) PANIC(MISC,"survive expected",NULL);
	_survive = kvp->getValueAsFloat();
	if((0.0F > _survive) || (_survive >= 1.0F))
		PANIC(MISC,"survive must be between 0 and 1",NULL);
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		kvp = ini.get(mutation_desc((MUTATION)i)); if(0 == kvp) PANIC(MISC,mutation_desc((MUTATION)i)," expected");
		_mutation_sum -= _mutation[(MUTATION)i];
		_mutation[(MUTATION)i] = kvp->getValueAsFloat();
		if(_mutation[(MUTATION)i] < 0.0F)
			PANIC(MISC,mutation_desc((MUTATION)i),"must be >= 0");
		_mutation_sum += _mutation[(MUTATION)i];
	}
	if(_mutation_sum <= 0.0F)
		PANIC(MISC,"sum mutation rate is invalid",NULL);
}

CReproduction *CReproduction::read_override_ini(INIFile &ini) /* loads values; if any values are
	specified, then a copy of this reproduction will be made; all unspecified values in the new
	class will be "inherited" from this one, and any specified values will be overriden.  If nothing
	is specified, then this will be returned */ {
	CReproduction *ret = this;
	KeyValuePair *kvp;
	int i;
	// crossover
	kvp = ini.get("crossover");
	if(0 != kvp) { // specified? override then
		if(!kvp->isValue(crossover_desc(_crossover))) { // different?
			ret = new CReproduction(*this); // create a copy to return
			ret->_crossover = crossover(kvp->getValueAsChar());
		}
	}
	// get crossover-type-specific vars
	switch(ret->_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				kvp = ini.get(random_component_desc((RANDOM_COMPONENT)i));
				if(0 != kvp) { // specified?  override then
					if(this == ret) { // nothing previously overriden?
						ret = new CReproduction(*this); // create a copy to return
						if(_crossover != ret->_crossover) {
							ret->_crossover_random_sum = 0.0F;
							for(int j=0; j<RANDOM_COMPONENT_LAST; j++)
								ret->_crossover_random[(RANDOM_COMPONENT)j] = 0.0F;
						}
					}
					ret->_crossover_random_sum -= ret->_crossover_random[(RANDOM_COMPONENT)i];
					ret->_crossover_random[(RANDOM_COMPONENT)i] = kvp->getValueAsFloat();
					ret->_crossover_random_sum += ret->_crossover_random[(RANDOM_COMPONENT)i];
					if(0.0F > _crossover_random[(RANDOM_COMPONENT)i])
						PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," cannot be negative");
				}
			}
			kvp = ini.get("adjust_addressing");
			if(0 != kvp) { // set?
				if(kvp->getValueAsBool(ret->_adjust_addressing) != ret->_adjust_addressing) { // changed?
					if(this == ret) { // nothing previously overriden?
						ret = new CReproduction(*this); // create a copy to return
					}
					ret->_adjust_addressing = !ret->_adjust_addressing; // and toggle it
				}
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	kvp = ini.get("survive");
	if(0 != kvp) { // specified?  override then
		if(this == ret) // crossover not already overriden?
			ret = new CReproduction(*this); // create a copy to return
		ret->_survive = kvp->getValueAsFloat();
		if((0.0F > _survive) || (_survive >= 1.0F))
			PANIC(MISC,"survive must be between 0 and 1",NULL);
	}
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		kvp = ini.get(mutation_desc((MUTATION)i));
		if(0 != kvp) { // specified?
			if(this == ret) { // crossover wasn't override?
				ret = new CReproduction(*this); // create a copy to return
			}
			ret->_mutation_sum -= ret->_mutation[(MUTATION)i];
			ret->_mutation[(MUTATION)i] = kvp->getValueAsFloat();
			if(ret->_mutation[(MUTATION)i] < 0.0F)
				PANIC(MISC,mutation_desc((MUTATION)i),"must be >= 0");
			ret->_mutation_sum += ret->_mutation[(MUTATION)i];
		}
	}
	if(ret->_mutation_sum <= 0.0F)
		PANIC(MISC,"sum mutation rate is invalid",NULL);
	// done
	return ret;
}

void CReproduction::write_ini(ostream &os) {
	int i;
	os << "crossover=" << crossover_desc(_crossover) << endl;
	// crossover type specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				os << random_component_desc((RANDOM_COMPONENT)i) << "=" << _crossover_random[(RANDOM_COMPONENT)i] << endl;
			}
			if(ADJUST_ADDRESSING_DEFAULT != _adjust_addressing) {
				os << "adjust_addressing=" << _adjust_addressing << endl;
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	os << "survive=" << _survive << endl;
	for(i=0; i<MUTATION_LAST; ++i) {
		os << mutation_desc((MUTATION)i) << '=' << _mutation[(MUTATION)i] << endl;
	}
}

void CReproduction::write_override_ini(ostream &os,const CReproduction *parent) {
	int i;
	if(this == parent) // not overriden?
		return;
	// crossover
	if(parent->_crossover != _crossover) {
		os << "crossover=" << crossover_desc(_crossover) << endl;
	}
	// crossover-type-specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				if((parent->_crossover != _crossover) || (parent->_crossover_random[(RANDOM_COMPONENT)i] != _crossover_random[(RANDOM_COMPONENT)i])) {
					os << random_component_desc((RANDOM_COMPONENT)i) << "=" << _crossover_random[(RANDOM_COMPONENT)i] << endl;
				}
			}
			if(parent->_adjust_addressing != _adjust_addressing) {
				os << "adjust_addressing=" << _adjust_addressing << endl;
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	if(parent->_survive != _survive) {
		os << "survive=" << _survive << endl;
	}
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		if(parent->_mutation[(MUTATION)i] != _mutation[(MUTATION)i]) {
			os << mutation_desc((MUTATION)i) << '=' << _mutation[(MUTATION)i] << endl;
		}
	}
}

CReproduction::RANDOM_COMPONENT CReproduction::crossover_random() const { // should there be something? prob controlled by _crossover_random[]
	const NUM x = CRand::frand(_crossover_random_sum);
	NUM y = 0.0F;
	do {
		for(int i=0; i<RANDOM_COMPONENT_LAST; i++) {
			y += _crossover_random[(RANDOM_COMPONENT)i];
			if(y > x) // found it?
				return (RANDOM_COMPONENT)i;
		}
		cerr << "WARNING: roulette past end in CReproduction:crossover_random()" << endl;
	} while(true); 
}

bool CReproduction::survive() const { // should a member of the current generation survive? prob controlled by _survive
	return (CRand::frand(1.0F) <= _survive);  // _survive is a percenhooke
}

CReproduction::MUTATION CReproduction::mutation() const { // should there be mutation? prob controlled by _entropy
	const NUM x = CRand::frand(_mutation_sum);
	NUM y = 0.0F;
	int i;
	do {
		for(i=0; i<MUTATION_LAST; ++i) {
			y += _mutation[(MUTATION)i];
			if(y > x) { // found it?
				return (MUTATION)i;
			}
		}
		cerr << "WARNING: roulette past end in CReproduction:mutation()" << endl;
	} while(true);
}

/* generates a completely random warrior using instruction biases in genus */
void CReproduction::generateCompletelyRandom(CChromosome &chromosome) {
	unsigned int i;
	//warrior.clear_info();
	// random length and start
	chromosome.rnd_len();
	if(chromosome.is_start())
		chromosome.set_start(CRand::irand(chromosome.len()));
	// generate random instructions
	chromosome.type()->freq()->set_chromosome(&chromosome);
	for(i=0; i<chromosome.len(); i++) {
		chromosome.type()->freq()->set_chromosome_index(i);
		chromosome.type()->freq()->suggest_instruction(chromosome.code(i),CInstGenerator::FORWARD);
	}
}

void CReproduction::mutate(CChromosome &chromosome,const int i) const { // apply mutation to an instruction in a gene
	//TODO: farm this out to CInstGen?
	// do mutation
	insn_t &in = chromosome.code(i); // easier on the eyes
try_again:
	switch(mutation()) {
		case CReproduction::MUT_OPCODE:
			in.set_opcode((OPCODE)CRand::irand(MODIFIER_LAST));
			break;
		case CReproduction::MUT_MODIFIER:
			if(1 < EFFECTIVE_MODIFIERS(in.opcode())) {
				MODIFIER mod;
				do {
					mod = EFFECTIVE_MODIFIER(in.opcode(),(MODIFIER)CRand::irand(MODIFIER_LAST));
				} while(in.modifier() == mod);
				in.set_modifier(mod);
			} else
				goto try_again;
			break;
		case CReproduction::MUT_ADDRMODE_A:
			in.set_addrmode_a((ADDRMODE)CRand::irand(ADDRMODE_LAST));
			break;
		case CReproduction::MUT_OPERAND_A:
			chromosome.type()->operands()->rnd(in,COperand::A);
			break;
		case CReproduction::MUT_ADDRMODE_B:
			in.set_addrmode_b((ADDRMODE)CRand::irand(ADDRMODE_LAST));
			break;
		case CReproduction::MUT_OPERAND_B:
			chromosome.type()->operands()->rnd(in,COperand::B);
			break;
		case CReproduction::MUT_NONE: // drop through
		default:
			break;
	}
	// checks?
	if(safety_checks) {
		if(!chromosome.code(i).valid(chromosome.type()->species()->kingdom()->coresize()))
			PANIC(MISC,"bad instruction",NULL);
	}
}

/* generates a new chromosome based upon two parents; despite the param naming, warriors are not sexed.
   chromosome stub must have been created (but contents are wiped) */   
void CReproduction::breed(CChromosome &chromosome,const CChromosome &mother,const CChromosome &father) {
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			breed_random(chromosome,mother,father);
			return;
	}
	PANIC(MISC,"bad crossover",NULL);
}

bool CReproduction::fit_enough(CWarrior &warrior) {
	// do checks
	if(!loops(warrior))
		return false;
	if(!runtest(warrior))
		return false;
	// if here, ok
	return true;
}

namespace { // anon

	void copy_instruction(CChromosome &chromosome,const int chromosome_idx,const CChromosome &parent,const int parent_idx,const field_t ajdustment,const field_t coresize) {
		insn_t &dest = chromosome.code(chromosome_idx);
		if(parent.len() > parent_idx) { // not off the end?
			const insn_t &src = parent.code(parent_idx);
			dest.in = src.in;
			dest.a = src.a;
			if(dest.a < chromosome_idx) { // need ajustment?
				dest.a += ajdustment;
				while(coresize <= dest.a) // fold?
					dest.a -= coresize;
			}
			dest.b = src.b;
			if(dest.b < chromosome_idx) { // need ajustment?
				dest.b += ajdustment;
				while(coresize <= dest.b) // fold?
					dest.b -= coresize;
			}
			if(parent_idx == parent.start())
				chromosome.set_start(chromosome_idx);
		} else { // off the end then
			chromosome.type()->freq()->set_chromosome_index(chromosome_idx);
			chromosome.type()->freq()->suggest_instruction(dest,CInstGenerator::FORWARD);
		}
	}
	
	void adjust_addressing_before(CChromosome &chromosome,const int current,const field_t coresize) {
		for(int i=0; i<current; i++) {
			if((chromosome.code(i).a >= current) && (chromosome.code(i).a < chromosome.len())) {
				if(coresize == ++chromosome.code(i).a) // inc, possibly fold?
					chromosome.code(i).a = 0; 
				
			}
			if((chromosome.code(i).b >= current) && (chromosome.code(i).b < chromosome.len())) {
				if(coresize == ++chromosome.code(i).b) // inc, possibly fold?
					chromosome.code(i).b = 0;
			}
		}
	}

} // anon namespace

void CReproduction::breed_random(CChromosome &chromosome,const CChromosome &mother,const CChromosome &father) {
	unsigned int i;
	bool parent_mother = true; // start with mother
	const field_t coresize = chromosome.type()->species()->kingdom()->coresize(); // cache it
	const CChromosome *parent = &mother;
	string &info = chromosome.info(); // straighter code
	info = "";
	// random length and start
	chromosome.rnd_len(); // a random length within bounds
	if(chromosome.is_start())
		chromosome.set_start(CRand::irand(chromosome.len())); // a default, might be inherited from parent
	// generate instructions
	chromosome.type()->freq()->set_chromosome(&chromosome);
	chromosome.clear();
	int cursor = 0, adjustment = 0;
	bool next = false;
	RANDOM_COMPONENT rnd = crossover_random();
	for(i=0; i<chromosome.len();) {
		switch(rnd) { // do crossover
			case RND_NONE:
				info += 'c';
				copy_instruction(chromosome,i,*parent,cursor,adjustment,coresize);
				cursor++;
				next = true;
				break;
			case RND_CROSSOVER:
				info += 'x';
				parent_mother = !parent_mother;
				parent = (parent_mother ? &mother: &father);
				rnd = RND_NONE;
				break;
			case RND_INSERT:
				info += 'i';
				if(0 == i) {
					rnd = RND_NONE;
					break;
				} else if(i < (chromosome.len()-1)) { // room?
					if(_adjust_addressing) adjustment++;
					chromosome.code(i).set_clear();
					copy_instruction(chromosome,i+1,*parent,cursor,adjustment,coresize);
					if(_adjust_addressing) adjust_addressing_before(chromosome,i,coresize);
					chromosome.type()->freq()->set_chromosome_index(i);
					chromosome.type()->freq()->suggest_instruction(chromosome.code(i),CInstGenerator::INSERT);
					cursor++;
					i++;
				} else {
					chromosome.type()->freq()->set_chromosome_index(i);
					chromosome.type()->freq()->suggest_instruction(chromosome.code(i),CInstGenerator::FORWARD);
				}
				next = true;
				break;
			case RND_REPLACE:
				info += 'r';
				if(0 == i) {
					rnd = RND_NONE;
					break;
				} else if(i < (chromosome.len()-1)) { // room?
					chromosome.code(i).set_clear();
					copy_instruction(chromosome,i+1,*parent,cursor+1,adjustment,coresize);
					chromosome.type()->freq()->set_chromosome_index(i);
					chromosome.type()->freq()->suggest_instruction(chromosome.code(i),CInstGenerator::INSERT);
					cursor++;
					i++;
				} else {
					chromosome.type()->freq()->set_chromosome_index(i);
					chromosome.type()->freq()->suggest_instruction(chromosome.code(i),CInstGenerator::FORWARD);
				}
				cursor++;
				next = true;
				break;
			case RND_DELETE:
				info += 'd';
				cursor++;
				if(0 < adjustment)
					adjustment--;
				rnd = RND_NONE;
				break;
			default:
				PANIC(MISC,"bad RANDOM_COMPONENT",NULL);
		}
		if(next) {
			mutate(chromosome,i);
			i++;
			rnd = crossover_random();
			next = false;
		}
	}
	// set info
	if(safety_checks)
		chromosome.check(coresize);
}

const char *CReproduction::mutation_desc(const CReproduction::MUTATION mutation) {
	if((0 <= mutation) && (mutation < MUTATION_LAST))
		return MUTATION_DESCS[mutation];
	return 0; // bad
}

const char *CReproduction::random_component_desc(const CReproduction::RANDOM_COMPONENT component) {
	if((0 <= component) && (component < RANDOM_COMPONENT_LAST))
		return RANDOM_COMPONENT_DESCS[component];
	return 0; // bad
}

const char *CReproduction::crossover_desc(const CReproduction::CROSSOVER crossover) {
	if((0 <= crossover) && (crossover < CROSSOVER_LAST))
		return CROSSOVER_DESCS[crossover];
	return 0; // bad
}

CReproduction::CROSSOVER CReproduction::crossover(const char *desc) {
	int i;
	for(i=0; i<CROSSOVER_LAST; ++i) {
		if(strcasecmp(desc,CROSSOVER_DESCS[(CROSSOVER)i]) == 0)
			return (CROSSOVER)i;
	}
	PANIC(MISC,"bad crossover type",NULL);
}
