#ifndef __CReproduction_HPP__
#define __CReproduction_HPP__

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "ini.hpp"
#include <iostream>

// a forward
class CWarrior;
class CChromosome;

class CReproduction { /* records the stats used for during breeding */
	//TODO: subclasses with factories?
	public:
		typedef float NUM;
		CReproduction();
		void read_ini(INIFile &ini);
		CReproduction *read_override_ini(INIFile &ini); /* loads values; if any values are
			specified, then a copy of this reproduction will be made; all unspecified values in the new
			class will be "inherited" from this one, and any specified values will be overriden.  If nothing
			is specified, then this will be returned */
		void write_ini(std::ostream &os);
		void write_override_ini(std::ostream &os,const CReproduction *parent);
		bool survive() const; // should a member of the current generation survive? prob controlled by _survive
		/* generates a completely random warrior using instruction biases in genus */
		void generateCompletelyRandom(CChromosome &chromosome);
		/* generates a new chromosome based upon two parents; despite the param naming, warriors are not sexed.
		   chromosome stub must have been created (but contents are wiped) */   
		void breed(CChromosome &chromosome,const CChromosome &mother,const CChromosome &father);
		static bool fit_enough(CWarrior &warrior); 
	protected:
		enum CROSSOVER { // what type of crossover are we using?
			CROSSOVER_RANDOM,
			CROSSOVER_LAST
		} _crossover; // what type of crossover strategy are we using?
		enum MUTATION { // where should mutation occur?
			MUT_NONE,
			MUT_OPCODE,
			MUT_MODIFIER,
			MUT_ADDRMODE_A,
			MUT_ADDRMODE_B,
			MUT_OPERAND_A,
			MUT_OPERAND_B,
			MUTATION_LAST
		};
		enum RANDOM_COMPONENT {
			RND_NONE,
			RND_CROSSOVER,
			RND_INSERT,
			RND_REPLACE,
			RND_DELETE,
			RANDOM_COMPONENT_LAST
		};
		NUM	_crossover_random[RANDOM_COMPONENT_LAST], // probability of crossover (used if CROSSOVER_RANDOM)
			_crossover_random_sum, // used if CROSSOVER_RANDOM
			_dat_padding, // percentatge of warrior that can be dat padding (used if CROSSOVER_BRANCH)
			_survive, // probability that a warrior will survive
			_mutation[MUTATION_LAST], // roulette-wheel of where mutation will occur
			_mutation_sum; // sum mutation
		static const char 
			*MUTATION_DESCS[MUTATION_LAST],
			*mutation_desc(const MUTATION mutation),
			*RANDOM_COMPONENT_DESCS[RANDOM_COMPONENT_LAST],
			*random_component_desc(const RANDOM_COMPONENT component),
			*CROSSOVER_DESCS[CROSSOVER_LAST],
			*crossover_desc(const CROSSOVER crossover);
		bool _fitness_bias,
			_adjust_addressing; // used by crossover_random to insert instructions smoothly
		// lookups
		static CROSSOVER crossover(const char *desc);
		// prob checks
		RANDOM_COMPONENT crossover_random() const; // should there be something? prob controlled by _crossover_random[]
		MUTATION mutation() const; // should there be mutation? prob controlled _mutation[]
		// shared routines used by breeding algorithms
		void mutate(CChromosome &chromosome,const int i) const; // apply mutation to an instruction in a gene
		// actual crossover implementations
		void breed_random(CChromosome &chromosome,const CChromosome &mother,const CChromosome &father);
};

#endif // __CReproduction_HPP__

