
#ifndef WARRIOR_HPP
#define WARRIOR_HPP

/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

/* nasty workaround, code needs refactoring */
#ifndef SPECIES_HPP 

	class CSpecies;
	class CReproduction;
	class COperand;
	class CGeneration;
	class CGenus;
	class CKingdom;
	class CChromosome;
	class CSpeciesFile;
	#include "exhaust.hpp"
	#include <iostream>

#endif // ifndef SPECIES_HPP

#include "fitness.hpp"
#include "generation.hpp"

class CExecTrail; // this is the way of the future!
 
class CWarrior: private warrior_t {
	friend class CChromosome;
	public:
		typedef unsigned int TUid;
		static const TUid NONE = 0;
		typedef struct {
			CWarrior::TUid src, dest;
			int win, lose, tie;
		} FIGHT_DATA;
		CWarrior(CSpecies *species); // create a new evolving warrior stub; assigns self a TUid */
		CWarrior(CSpecies *species,TUid mother,TUid father); // create a new evolving warrior stub; assigns parents, assigns self a TUid */
		CWarrior(CSpecies *species,const TUid uid); // create a new evolving warrior stub; assigns specified TUid */
		CWarrior(CSpecies *species,const std::string &rc_filename); /* create and load a benchmark warrior */
		~CWarrior();
		void clear();
		unsigned len() const { return warrior_t::len; }
		unsigned start_idx() const { return start; }
		void set_start(const unsigned i);
		const insn_t &code(const unsigned i) const;
		//const insn_t &operator[](unsigned i) const;
		const std::string &filename() const { return _filename; }
		const std::string &info() const { return _info; }
		void set_info(std::string &info) { _info = info; }
		void set_info(const char *info) { _info = info; }
		void clear_info() { _info = ""; }
		bool is_evolving() const { return _evolving; }
		bool is_bench() const { return !_evolving; }
		TUid uid() const { return _uid; }
		TUid mother() const { return _mother; }
		TUid father() const { return _father; }
		void set_parents(const int mother,const int father) { _mother = mother; _father = father; }
		void set_uid(const TUid uid);
		bool is_well_formed() const; /* returns if this is *potentially* an ok warrior (basic init checks really) */
		bool equals(const CWarrior &other) const; /* truely equals */
		CGeneration::NUM generation() const { return _generation; }
		void set_generation(const CGeneration::NUM generation) { _generation = generation; }
		void ready(); /* this will blank all fitness metrics preceeding the fighting of a generation */
		bool fought() const { return _fought; } /* has this already fought? (resume support) */
		void fought(bool has) { _fought = has; } /* a setter (resume support) */
		void load(const int pos) const;
		int fight(CWarrior &enemy,FIGHT_DATA &data,const bool fight=true); /* populates data and returns number of rounds fought */
		int fight(const CWarrior &bench,const int rounds,const int win,const int tie,const int lose); /* returns score */ 
		void score(const CFitness::CAT cat,const CFitness::RESULT res,const CWarrior &enemy,int rounds=1); /* this will adjust the fitness metrics to account for this result */
		CFitness::NUM score() const { return _fitness_score; }
		CSpecies *species() const { return _species; }
		CGenus *genus() const;
		CKingdom *kingdom() const;
		COperand *operands() const;
		CExecTrail *exec_trail() const;
		void code_check() const;
		CFitness::NUM benchmark() const;
		bool is_benchmarked() const { return (0 < _benchmark_count); }
		void incorporate_benchmark(CFitness::NUM score); // incorporate this benchmark
		int benchmark_count() const { return _benchmark_count; }
		void set_benchmark(CFitness::NUM sum,int count) { _benchmark_count = count; _benchmark_sum = sum; }
		void rc(std::ostream &os); // prints an .rc to this stream
		void red(std::ostream &os,const char *note = 0); // prints a headed .red file to this stream
		CChromosome *chromosome(unsigned int i) const;
	protected:
		CSpecies *_species;
		CGeneration::NUM _generation;
		bool _evolving, _fought;
		TUid _uid,
			_mother, _father,
			next_uid(); /* assigns self a new uid, and returns it */
		std::string _filename, _info;
		CFitness::NUM _fitness_score, /* these is a score of the fitness of this warrior; attacks and defences are weighed differently */
			_benchmark_sum;
		CExecTrail *_exec_trail;
		unsigned int _benchmark_count;
		CChromosome **_chromosome; // _chromosome[Species()->num_chromosomes()] details length of each chromosome in the warrior
		void create_chromosomes();
		void calc_len();
};

#endif // ifndef WARRIOR_HPP
