/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */
 
#include "compress.h"

#include <zlib.h>

static panic(const char *reason) {
	fprintf(stderr,"\ncompress ERROR: %s\n",reason);
	exit(1);
}

static unsigned long long physical_bytes, logical_bytes;
// metrics
void init_compress_metrics() {
	physical_bytes = 0;
	logical_bytes = 0;
}

unsigned long long get_physical_bytes() { return physical_bytes; }
unsigned long long get_logical_bytes() { return logical_bytes; }

static comp_buf_t temp, check;

void block_compress(comp_buf_t *src) {
	int i, zret;
	if(src->len == 0) return;
	// compress it
	if(src->len > COMPRESS_BUFFER) panic("c1");
	temp.len = COMPRESS_SAFE_BUFFER;
	zret = compress(temp.buf,&temp.len,src->buf,src->len); // try to compress
	if(zret != Z_OK) {
		fprintf(stderr,"Z_ERROR: %s\n",zError(zret));
		panic("c2");
	}
	// write it
	if(1 != fwrite(&temp.len,sizeof(uLongf),1,src->f)) panic("c3"); // block len
	if(temp.len != fwrite(temp.buf,sizeof(Bytef),temp.len,src->f)) panic("c4"); // compressed data
	// done
	//fflush(src->f);
	physical_bytes += temp.len;
	logical_bytes += src->len;
}

void flush_compressed(comp_buf_t *src) {
	block_compress(src);
	src->len = 0;
}

void write_compressed(comp_buf_t *src,Bytef ch) {
	src->buf[src->len++] = ch;
	if(src->len == COMPRESS_BUFFER)
		flush_compressed(src);
}

void block_decompress(comp_buf_t *dest) {
	int zret;
	// read block
	if(1 != fread(&temp.len,sizeof(uLongf),1,dest->f)) { // block len? not successful suggests eof
		dest->len = 0;
		return;
	}
	if(temp.len > COMPRESS_SAFE_BUFFER) panic("d1");
	if(temp.len != fread(temp.buf,sizeof(Bytef),temp.len,dest->f)) panic("d2"); // compressed data
	// decompress it
	dest->len = COMPRESS_SAFE_BUFFER;
	zret = uncompress(dest->buf,&dest->len,temp.buf,temp.len);
	if(zret != Z_OK) {
		fprintf(stderr,"Z_ERROR: %s\n",zError(zret));
		panic("d3");
	}
	// done
	physical_bytes += temp.len;
	logical_bytes += dest->len;
}

